-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.09.20					---
--- Version: 0.4 [2012.01.18]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/guildnotechanges
--- WoWInterface	http://www.wowinterface.com/downloads/info20322-GuildNoteChanges.html

local NAME = ...
local VERSION = 0.4

local format = format
local GetGuildRosterInfo = GetGuildRosterInfo

local db, viewOfficer

	---------------
	--- Caching ---
	---------------

local cache = {}

local function GetClassColor(class)
	if cache[class] then
		return cache[class]
	else
		local color = RAID_CLASS_COLORS[class]
		cache[class] = format("%02X%02X%02X", color.r*255, color.g*255, color.b*255)
		return cache[class]
	end
end

	-------------
	--- Frame ---
	-------------

local delay, cd = 0, 0
local noop = function() end

local f = CreateFrame("Frame")

-- delay initialization, because some guild API does not yet
-- return the correct information (even after ADDON_LOADED)
function f.OnUpdate(self, elapsed)
	delay = delay + elapsed
	if delay > 5 then
		if IsInGuild() then
			local realm = GetRealmName()
			local guild = GetGuildInfo("player")
			GuildNoteChangesDB[realm] = GuildNoteChangesDB[realm] or {}
			GuildNoteChangesDB[realm][guild] = GuildNoteChangesDB[realm][guild] or {}
			db = GuildNoteChangesDB[realm][guild]
			viewOfficer = CanViewOfficerNote()
			self:RegisterEvent("GUILD_ROSTER_UPDATE")
		end
		self:SetScript("OnUpdate", noop)
	end
end

function f.OnEvent(self, event, ...)
	if event == "ADDON_LOADED" and ... == NAME then
		GuildNoteChangesDB = GuildNoteChangesDB or {}
		GuildNoteChangesDB.version = VERSION
		self:UnregisterEvent("ADDON_LOADED")
		self:SetScript("OnUpdate", f.OnUpdate)
	elseif event == "GUILD_ROSTER_UPDATE" then
		if time() > cd then -- throttle
			cd = time() + 2
			for i = 1, GetNumGuildMembers() do
				local name, _, _, _, _, _, publicNote, officerNote, _, _, englishClass = GetGuildRosterInfo(i)
				db[name] = db[name] or {}
				local classColor = GetClassColor(englishClass)
				if db[name][1] and db[name][1] ~= publicNote then
					local text
					if db[name][1] == "" then
						text = publicNote
					else
						text = format("%s |cff%s->|r %s", db[name][1], classColor, publicNote == "" and "|cffFF0000N/A|r" or publicNote)
					end
					print(format("|cff%s|Hplayer:%s|h[%s]|h|r %s", classColor, name, name, text))
				end
				if viewOfficer then
					if db[name][2] and db[name][2] ~= officerNote then
						local text
						if db[name][2] == "" then
							text = officerNote
						else
							text = format("%s |cff%s->|r %s", db[name][2], classColor, officerNote == "" and "|cffFF0000N/A|r" or officerNote)
						end
						print(format("|cffFF7F00[%s]|r |cff%s|Hplayer:%s|h[%s]|h|r %s", OFFICER_NOTE_COLON, classColor, name, name, text))
					end
					db[name][2] = officerNote
				end
				db[name][1] = publicNote
			end
		end
	end
end

f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", f.OnEvent)